// TI File $Revision: /main/2 $
// Checkin $Date: July 30, 2009   18:44:57 $
//###########################################################################
//
// FILE:    Example_2833xFlash.c
//
// TITLE:   DSP2833x ePWM Timer Interrupt From Flash Example.
//
// ASSUMPTIONS:
//
//    This program requires the DSP2833x header files.
//
//    As supplied, this project is configured for "boot to FLASH"
//    operation.  The 2833x Boot Mode table is shown below.
//    For information on configuring the boot mode of an eZdsp,
//    please refer to the documentation included with the eZdsp,
//
//       $Boot_Table:
//
//         GPIO87   GPIO86     GPIO85   GPIO84
//          XA15     XA14       XA13     XA12
//           PU       PU         PU       PU
//        ==========================================
//            1        1          1        1    Jump to Flash <- "boot to Flash"
//            1        1          1        0    SCI-A boot
//            1        1          0        1    SPI-A boot
//            1        1          0        0    I2C-A boot
//            1        0          1        1    eCAN-A boot
//            1        0          1        0    McBSP-A boot
//            1        0          0        1    Jump to XINTF x16
//            1        0          0        0    Jump to XINTF x32
//            0        1          1        1    Jump to OTP
//            0        1          1        0    Parallel GPIO I/O boot
//            0        1          0        1    Parallel XINTF boot
//            0        1          0        0    Jump to SARAM
//            0        0          1        1    Branch to check boot mode
//            0        0          1        0    Boot to flash, bypass ADC cal
//            0        0          0        1    Boot to SARAM, bypass ADC cal
//            0        0          0        0    Boot to SCI-A, bypass ADC cal
//                                              Boot_Table_End$
//
// DESCRIPTION:
//
//    This example runs the ePWM interrupt example from flash.
//
//    1) Build the project
//    2) Flash the .out file into the device.
//    3) Set the hardware jumpers to boot to Flash
//    4) Load the project, (in CCSv4 - loads directly to Flash)
//       and watch variables in watch window while running.
//
//    Steps that were taken to convert the ePWM example from RAM
//    to Flash execution:
//
//    - Change the linker cmd file to reflect the flash memory map.
//    - Make sure any initialized sections are mapped to Flash.
//      In SDFlash utility this can be checked by the View->Coff/Hex
//      status utility. Any section marked as "load" should be
//      allocated to Flash.
//    - Make sure there is a branch instruction from the entry to Flash
//      at 0x33FFF6 to the beginning of code execution. This example
//      uses the DSP2833x_CodeStartBranch.asm file to accomplish this.
//    - Set boot mode Jumpers to "boot to Flash"
//    - For best performance from the flash, modify the waitstates
//      and enable the flash pipeline as shown in this example.
//      Note: any code that manipulates the flash waitstate and pipeline
//      control must be run from RAM. Thus these functions are located
//      in their own memory section called ramfuncs.
//
//    - NOTE: this example runs on the 28335 device. To run the example
//      on other 2833x device derivatives, change out the F28335.cmd linker
//      command file with the desired device linker command file.
//
//
//    ePwm1 Interrupt will run from RAM and puts the flash into sleep mode
//    ePwm2 Interrupt will run from RAM and puts the flash into standby mode
//    ePWM3 Interrupt will run from FLASH
//
//    As supplied:
//
//    All timers have the same period
//    The timers are started sync'ed
//    An interrupt is taken on a zero event for each ePWM timer
//
//       ePWM1: takes an interrupt every event
//       ePWM2: takes an interrupt every 2nd event
//       ePWM3: takes an interrupt every 3rd event
//
//    Thus the Interrupt count for ePWM1, ePWM4-ePWM6 should be equal
//    The interrupt count for ePWM2 should be about half that of ePWM1
//    and the interrupt count for ePWM3 should be about 1/3 that of ePWM1
//
//          Watch Variables:
//                 EPwm1TimerIntCount
//                 EPwm2TimerIntCount
//                 EPwm3TimerIntCount
//
//                 Toggle GPIO32 while in the background loop.
//
//###########################################################################
// $TI Release: 2833x/2823x Header Files V1.32 $
// $Release Date: June 28, 2010 $
//###########################################################################


#include "DSP28x_Project.h"     	// Device Headerfile and Examples Include File
#include "init.h"					//"Configuration/init.h"
//#include "CommunicationCPU/SpiInterrupt.h"
//#include "SignalsGeneration.h"		// "SignalsGeneration/SignalsGeneration.h"
#include "SignalsFunctions.h"		//"SignalsGeneration/SignalsFunctions.h"
//#include "BootLoader/BootLoaderFunctions.h"

//-------------------------------------------------------------------------------------------------------------
//------------------------------------------- Variables -------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------

enum type_signal_state signal_state = sinus;					// Variable that describes the current state of the trapeze

// Sinus
int sample_number = 0;											// Number of the current sample of the look-up table
int increment = 1;												// Increment in the look-up table between to samples
unsigned int magnitude_int = 0;									// Magnitude of the half-sinus (20 mA = 65535)
double magnitude_double = 0.5;									// Magnitude of the half-sinus (20 mA = 1.0)
int timer_period = 0;											// Timer 0 period for the sinus generation
extern unsigned int HalfSinus_LookUpTable [2000];				// Look-up table for a half sinus with 2000 points
unsigned int HalfSinus_ScaledLookUpTable [2000];				// Scaled Look-up table for a half sinus with 2000 points
int angle_min = 889;											// lower angular limit for the hand-shake signal
int angle_max = 1111;											// upper angular limit for the hand-shake signal

// Trapeze
int current_dao = 0;											// Integer to memorize the previous value of the Dynamic Analog Output (DAO)
unsigned int segment_counter=0;                        			// Number of the currently generated trapeze segment
unsigned int current_segment;                        			// Number of the currently generated trapeze segment
int current_increment_value=0;                 					// Increment of the currently generated segment
unsigned int current_plateau_duration=0;          				// Number of interrupt to generate the current plateau
unsigned int plateau_duration_counter=0;						// Count the number of interrupt passed to generate the plateau
unsigned int current_plateau_value=0;          					// Value of the plateau of the currently generated segment
//unsigned int plateau_max_duration[10]={0};       				// Table for the maximal duration of a plateau
int slope_increment[10]={0};       								// Table for the increments during the slope
unsigned int plateau_value[10]={0};								// Table for the value of the plateau
unsigned int plateau_duration[10]={0};    				   		// Table for the duration of a plateau
unsigned int slope_time_interval[10]={100};						// Table for the period of timer0 interrupt
unsigned int slope_counter_limit[10]={0};						// Table for the limit of the slope increment counter
unsigned int slope_counter = 0;									// Counter for the slope duration
unsigned int max_segment = 9;									// maximum number of segment
int mantissa_value = 0;											// For the translation from the mantissa format
int mantissa_exponent = 0;										// Idem

// Communication - "DSP-registers"
unsigned int reg0 = 0;											// For sinus <=> Magnitude on 16 bits, For trapeze <=> value of the plateau
unsigned int reg1 = 0;											// For sinus <=> increment (1,2,4,5,8 or 10), For trapeze <=> period of timer0
int reg2 = 0;													// For sinus <=> period of timer zero, For trapeze, increment between to samples
int reg3 = 0;													// Only for trapeze <=> plateau duration (mantissa format: 12 bits for value, 4 bits for exponent)
int reg4 = 0;													// Meaning of the value in the other register
int reg5 = 0;													// For both types: writting a value in this register a is soft synchro
int reg_out = 0;												// Value to send to the uC (0x55 <=> a curve is under generation, 0xAA, a curve is ready to be generated)
char received_word = 0;											// Number of received bytes for the current message (address + value)
int address = 0;												// Received address
int value = 0;													// Received value
int received_data = 0;											// "raw" received data
int dummy_interrupt = 1;										// for differenciate the first false interrupt
long IntermediaryPlateauDuration = 0;							// Intermediary variable to compute the plateau duration
int check_sum  = 0;												// check sum to verify the value in all registers at once
int check_sum_LOW = 0;											// intermediary variable
int check_sum_HIGH = 0;											// intermediary variable

// Communication UART
char LowChar = 0;												// 0 if the current character is the lower byte
																// 1 if the current character is the upper byte
char ReceivedChar = 0;											// The received 8-bit character
char ConvertedChar = 0;											// Converted value from Ascii
int AddressEeprom = 0;											// Address in the eeprom
char InvalidCharReceived = 0;									// Check if an invalid character has been received
char *msg;														// message to be send to the PC
char ReadBuffer = 0;											// Value of the buffer at the beginning of the interrupt


// General
int software_version = 10;										// software version, to be read 1.0
																// !!!! If modified, please change line 103 of main. !!!!!


/*

// Configure which ePWM timer interrupts are enabled at the PIE level:
// 1 = enabled,  0 = disabled
#define PWM1_INT_ENABLE  1
#define PWM2_INT_ENABLE  1
#define PWM3_INT_ENABLE  1

// Configure the period for each timer
#define PWM1_TIMER_TBPRD   0x1FFF
#define PWM2_TIMER_TBPRD   0x1FFF
#define PWM3_TIMER_TBPRD   0x1FFF

// Make this long enough so that we can see an LED toggle
#define DELAY 1000000L

// Functions that will be run from RAM need to be assigned to
// a different section.  This section will then be mapped using
// the linker cmd file.
#pragma CODE_SECTION(epwm1_timer_isr, "ramfuncs");
#pragma CODE_SECTION(epwm2_timer_isr, "ramfuncs");

// Prototype statements for functions found within this file.
interrupt void epwm1_timer_isr(void);
interrupt void epwm2_timer_isr(void);
interrupt void epwm3_timer_isr(void);
void InitEPwmTimer(void);

// Global variables used in this example
Uint32  EPwm1TimerIntCount;
Uint32  EPwm2TimerIntCount;
Uint32  EPwm3TimerIntCount;
Uint32  LoopCount;

*/

// These are defined by the linker (see F28335.cmd)
extern Uint16 RamfuncsLoadStart;
extern Uint16 RamfuncsLoadEnd;
extern Uint16 RamfuncsRunStart;

void main(void)
{

	//init();									// Initialization functions

/*

// Step 1. Initialize System Control:
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the DSP2833x_SysCtrl.c file.
   InitSysCtrl();

// Step 2. Initalize GPIO:
// This example function is found in the DSP2833x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
// InitGpio();  // Skipped for this example


// Step 3. Clear all interrupts and initialize PIE vector table:
// Disable CPU interrupts
   DINT;

// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
// This function is found in the DSP2833x_PieCtrl.c file.
   InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags:
   IER = 0x0000;
   IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in DSP2833x_DefaultIsr.c.
// This function is found in DSP2833x_PieVect.c.
   InitPieVectTable();

// Interrupts that are used in this example are re-mapped to
// ISR functions found within this file.
   EALLOW;  // This is needed to write to EALLOW protected registers
   PieVectTable.EPWM1_INT = &epwm1_timer_isr;
   PieVectTable.EPWM2_INT = &epwm2_timer_isr;
   PieVectTable.EPWM3_INT = &epwm3_timer_isr;
   EDIS;    // This is needed to disable write to EALLOW protected registers

// Step 4. Initialize all the Device Peripherals:
// This function is found in DSP2833x_InitPeripherals.c
// InitPeripherals();  // Not required for this example
   InitEPwmTimer();    // For this example, only initialize the ePWM Timers

// Step 5. User specific code, enable interrupts:

 */

// Copy time critical code and Flash setup code to RAM
// This includes the following ISR functions: epwm1_timer_isr(), epwm2_timer_isr()
// epwm3_timer_isr and and InitFlash();
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the F28335.cmd file.
   MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
   InitFlash();

/*

// Initalize counters:
   EPwm1TimerIntCount = 0;
   EPwm2TimerIntCount = 0;
   EPwm3TimerIntCount = 0;
   LoopCount = 0;

// Enable CPU INT3 which is connected to EPWM1-3 INT:
   IER |= M_INT3;

// Enable EPWM INTn in the PIE: Group 3 interrupt 1-3
   PieCtrlRegs.PIEIER3.bit.INTx1 = PWM1_INT_ENABLE;
   PieCtrlRegs.PIEIER3.bit.INTx2 = PWM2_INT_ENABLE;
   PieCtrlRegs.PIEIER3.bit.INTx3 = PWM3_INT_ENABLE;

// Enable global Interrupts and higher priority real-time debug events:
   EINT;   // Enable Global interrupt INTM
   ERTM;   // Enable Global realtime interrupt DBGM

// Step 6. IDLE loop. Just sit and loop forever (optional):
   EALLOW;
   GpioCtrlRegs.GPBMUX1.bit.GPIO32 = 0;
   GpioCtrlRegs.GPBDIR.bit.GPIO32 = 1;
   EDIS;

   for(;;)
   {
       // This loop will be interrupted, so the overall
       // delay between pin toggles will be longer.
       DELAY_US(DELAY);
       LoopCount++;
       GpioDataRegs.GPBTOGGLE.bit.GPIO32 = 1;
   }

 */

	SpiaRegs.SPITXBUF = software_version;	// put the software version in the transmit buffer of the SPId

	GpioDataRegs.GPASET.bit.GPIO21 = 1; 	// Turn the green led on to show the DSP has booted correctly...

	*DynamicAnalogOutput = 0xFFFF;			// init of the dynamic output to 0 (inverted polarity in hardware)
											// Allows the dynamic output to be 0 during the DSP start-up

	// Check if the DSP is on the UART bus
//	msg = "\r\nDSP present on the UART bus: version 1.0 \n\0";
//	scia_msg(msg);

	ResetTrapeze();							// init the different parameter for the trapeze
	ResetSinus();							// init the different parameter for the sinus

	//------------------------ BEGIN OF DEBUG ---------------------------------------------------
	//ResetEeprom();							// reset the eeprom
	//------------------------ END OF DEBUG ------------------------------------------------------------

	// Beginning of the infinite loop
	while(1)
	{

	}

	// Should never reach this point


}

/*
void InitEPwmTimer()
{

   EALLOW;
   SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;      // Stop all the TB clocks
   EDIS;

   // Setup Sync
   EPwm1Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;  // Pass through
   EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;  // Pass through
   EPwm3Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN;  // Pass through

   // Allow each timer to be sync'ed

   EPwm1Regs.TBCTL.bit.PHSEN = TB_ENABLE;
   EPwm2Regs.TBCTL.bit.PHSEN = TB_ENABLE;
   EPwm3Regs.TBCTL.bit.PHSEN = TB_ENABLE;

   EPwm1Regs.TBPHS.half.TBPHS = 100;
   EPwm2Regs.TBPHS.half.TBPHS = 200;
   EPwm3Regs.TBPHS.half.TBPHS = 300;

   EPwm1Regs.TBPRD = PWM1_TIMER_TBPRD;
   EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;    // Count up
   EPwm1Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;     // Select INT on Zero event
   EPwm1Regs.ETSEL.bit.INTEN = PWM1_INT_ENABLE;  // Enable INT
   EPwm1Regs.ETPS.bit.INTPRD = ET_1ST;           // Generate INT on 1st event


   EPwm2Regs.TBPRD = PWM2_TIMER_TBPRD;
   EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;     // Count up
   EPwm2Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;      // Enable INT on Zero event
   EPwm2Regs.ETSEL.bit.INTEN = PWM2_INT_ENABLE;   // Enable INT
   EPwm2Regs.ETPS.bit.INTPRD = ET_2ND;            // Generate INT on 2nd event


   EPwm3Regs.TBPRD = PWM3_TIMER_TBPRD;
   EPwm3Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;     // Count up
   EPwm3Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;      // Enable INT on Zero event
   EPwm3Regs.ETSEL.bit.INTEN = PWM3_INT_ENABLE;   // Enable INT
   EPwm3Regs.ETPS.bit.INTPRD = ET_3RD;            // Generate INT on 3rd event

   EALLOW;
   SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;         // Start all the timers synced
   EDIS;


}

// This ISR MUST be executed from RAM as it will put the Flash into Sleep
// Interrupt routines uses in this example:
interrupt void epwm1_timer_isr(void)
{

   // Put the Flash to sleep
   FlashRegs.FPWR.bit.PWR = FLASH_SLEEP;

   EPwm1TimerIntCount++;

   // Clear INT flag for this timer
   EPwm1Regs.ETCLR.bit.INT = 1;

   // Acknowledge this interrupt to receive more interrupts from group 3
   PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

// This ISR MUST be executed from RAM as it will put the Flash into Standby
interrupt void epwm2_timer_isr(void)
{
   EPwm2TimerIntCount++;

    // Put the Flash into standby
    FlashRegs.FPWR.bit.PWR = FLASH_STANDBY;

   // Clear INT flag for this timer
   EPwm2Regs.ETCLR.bit.INT = 1;

   // Acknowledge this interrupt to receive more interrupts from group 3
   PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

interrupt void epwm3_timer_isr(void)
{
   Uint16 i;

   EPwm3TimerIntCount++;

    // Short Delay to simulate some ISR Code
    for(i = 1; i < 0x01FF; i++) {}

   // Clear INT flag for this timer
   EPwm3Regs.ETCLR.bit.INT = 1;

   // Acknowledge this interrupt to receive more interrupts from group 3
   PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}

*/


//===========================================================================
// No more.
//===========================================================================
